
from pathlib import Path
from os import environ
import regex


def env_to_bool(key, default) -> bool:
    v = environ.get(key, default)
    if isinstance(v, bool):
        return v
    v = v.lower()
    if v == 'none':
        return default
    r = v in ['true', '1', 'on', 'yes', 'active']
    return r


def env_to_list(key, default) -> list:
    v = environ.get(key, default)
    if isinstance(v, list):
        return v
    if v is None:
        return default
    v = regex.split('; *', v)
    return v


def env_to_path(key, default, create=False) -> Path:
    v = environ.get(key, default)
    if v is None:
        return default
    v = Path(v)
    if create and not v.exists():
        v.mkdir(parents=True, exist_ok=True)
    return v





class callCounter:
    """
    Count how many times a function called
    """

    def __init__(self, func):
        self.func = func
        self.counter = 0
        self.initCounter = 0

    def __call__(self, *args, **kwargs):
        self.counter = self.counter + 1
        self.initCounter = self.initCounter + 1
        return self.func(*args, **kwargs)

    def reset(self):
        self.counter = 0

    def setCounter(self, num):
        self.counter = num

    def get(self):
        return self.counter

    def getFullCount(self):
        return self.initCounter


class return_bool_counter(object):
    """
    Count how many times a function return true and false
    """
    def __init__(self, func):
        self.func = func
        self.true_counter = 0
        self.false_counter = 0

    def __call__(self, *args, **kwargs):
        r = self.func(*args, **kwargs)
        if isinstance(r, bool):
            if r:
                self.true_counter += 1
            else:
                self.false_counter += 1
        elif isinstance(r, list | tuple):
            if True in r:
                self.true_counter += 1
            else:
                self.false_counter += 1
        return r

    def reset(self):
        self.true_counter = 0
        self.false_counter = 0

    def get_true(self):
        return self.true_counter

    def get_false(self):
        return self.false_counter


class CachedProperty:
    def __init__(self, func):
        self.func = func

    def __get__(self, instance, owner):
        if instance is None:
            return self
        value = instance.__dict__[self.func.__name__] = self.func(instance)
        return value

